@@ -15,7 +15,7 @@
 #include "common/exception.h"
 #include "common/thread.h"
 #include "common/tasking.h"
-#include "math.h"
+#include "common/math.h"
 #include "input_reorder_ispc.h" // ispc::Memory, ispc::Image
 
 namespace oidn {
@@ -25,7 +25,8 @@ namespace oidn {
   using dnnl::impl::parallel_nd;
   using dnnl::impl::memory_desc_matches_tag;
 
-  inline size_t getFormatBytes(Format format)
+  // Returns the size of the format in bytes
+  inline size_t getFormatSize(Format format)
   {
     switch (format)
     {
