@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019 Intel Corporation
+* Copyright 2019-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -28,7 +28,6 @@
 #include <cassert>
 #include <cmath>
 #include <iostream>
-#include <numeric>
 #include <stdexcept>
 
 #include "dnnl.hpp"
@@ -37,11 +36,6 @@
 
 using namespace dnnl;
 
-memory::dim product(const memory::dims &dims) {
-    return std::accumulate(dims.begin(), dims.end(), (memory::dim)1,
-            std::multiplies<memory::dim>());
-}
-
 void simple_net() {
     using tag = memory::format_tag;
     using dt = memory::data_type;
@@ -57,7 +51,6 @@ void simple_net() {
 
     // float data type is used for user data
     std::vector<float> net_src(batch * 3 * 227 * 227);
-    std::vector<float> net_dst(batch * 96 * 27 * 27);
 
     // initializing non-zero values for src
     for (size_t i = 0; i < net_src.size(); ++i)
@@ -85,13 +78,17 @@ void simple_net() {
         conv_bias[i] = sinf((float)i);
 
     // create memory for user data
-    auto conv_user_src_memory = memory(
-            {{conv_src_tz}, dt::f32, tag::nchw}, cpu_engine, net_src.data());
+    auto conv_user_src_memory
+            = memory({{conv_src_tz}, dt::f32, tag::nchw}, cpu_engine);
+    write_to_dnnl_memory(net_src.data(), conv_user_src_memory);
+
     auto conv_user_weights_memory
-            = memory({{conv_weights_tz}, dt::f32, tag::oihw}, cpu_engine,
-                    conv_weights.data());
-    auto conv_user_bias_memory = memory(
-            {{conv_bias_tz}, dt::f32, tag::x}, cpu_engine, conv_bias.data());
+            = memory({{conv_weights_tz}, dt::f32, tag::oihw}, cpu_engine);
+    write_to_dnnl_memory(conv_weights.data(), conv_user_weights_memory);
+
+    auto conv_user_bias_memory
+            = memory({{conv_bias_tz}, dt::f32, tag::x}, cpu_engine);
+    write_to_dnnl_memory(conv_bias.data(), conv_user_bias_memory);
 
     // create memory descriptors for bfloat16 convolution data w/ no specified
     // format tag(`any`)
@@ -114,7 +111,7 @@ void simple_net() {
     } catch (error &e) {
         if (e.status == dnnl_unimplemented)
             throw example_allows_unimplemented {
-                    "DNNL does not have bf16 convolution implementation "
+                    "oneDNN does not have bf16 convolution implementation "
                     "that supports this system.\n"
                     "Please refer to the developer guide for details."};
 
@@ -214,8 +211,8 @@ void simple_net() {
     memory::dims pool_padding = {0, 0};
 
     // create memory for pool dst data in user format
-    auto pool_user_dst_memory = memory(
-            {{pool_dst_tz}, dt::f32, tag::nchw}, cpu_engine, net_dst.data());
+    auto pool_user_dst_memory
+            = memory({{pool_dst_tz}, dt::f32, tag::nchw}, cpu_engine);
 
     // create pool dst memory descriptor in format any for bfloat16 data type
     auto pool_dst_md = memory::desc({pool_dst_tz}, dt::bf16, tag::any);
@@ -258,8 +255,9 @@ void simple_net() {
         net_diff_dst[i] = sinf((float)i);
 
     // create memory for user diff dst data stored in float data type
-    auto pool_user_diff_dst_memory = memory({{pool_dst_tz}, dt::f32, tag::nchw},
-            cpu_engine, net_diff_dst.data());
+    auto pool_user_diff_dst_memory
+            = memory({{pool_dst_tz}, dt::f32, tag::nchw}, cpu_engine);
+    write_to_dnnl_memory(net_diff_dst.data(), pool_user_diff_dst_memory);
 
     // Backward pooling
     // create memory descriptors for pooling
@@ -355,14 +353,11 @@ void simple_net() {
 
     // Backward convolution with respect to weights
     // create user format diff weights and diff bias memory for float data type
-    std::vector<float> conv_user_diff_weights_buffer(product(conv_weights_tz));
-    std::vector<float> conv_diff_bias_buffer(product(conv_bias_tz));
 
     auto conv_user_diff_weights_memory
-            = memory({{conv_weights_tz}, dt::f32, tag::nchw}, cpu_engine,
-                    conv_user_diff_weights_buffer.data());
-    auto conv_diff_bias_memory = memory({{conv_bias_tz}, dt::f32, tag::x},
-            cpu_engine, conv_diff_bias_buffer.data());
+            = memory({{conv_weights_tz}, dt::f32, tag::nchw}, cpu_engine);
+    auto conv_diff_bias_memory
+            = memory({{conv_bias_tz}, dt::f32, tag::x}, cpu_engine);
 
     // create memory descriptors for bfloat16 convolution data
     auto conv_bwd_src_md = memory::desc({conv_src_tz}, dt::bf16, tag::any);
