@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2019 Intel Corporation
+* Copyright 2017-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -98,7 +98,8 @@ static const impl_list_map_t regular_impl_list_map {
 
         jit_uni_reorder_create,
 
-        REG_SR(f32, nchw, bf16, nChw16c, fmt_order::keep),
+        REG_SR_BIDIR(f32, any, bf16, nChw16c),
+        REG_SR_BIDIR(f32, any, bf16, nCdhw16c),
 
         REG_SR(f32, oihw, bf16, OIhw8i16o2i, fmt_order::keep),
         REG_SR(f32, goihw, bf16, gOIhw8i16o2i, fmt_order::keep),
@@ -244,6 +245,7 @@ static const impl_list_map_t regular_impl_list_map {
         REG_SR_BIDIR(f32, any, f32, Odhwi16o),
         REG_SR_BIDIR(f32, any, f32, OIdhw16o16i),
         REG_SR_BIDIR(f32, any, f32, OIdhw16i16o),
+        REG_SR_BIDIR(f32, any, f32, IOdhw16o16i),
 
         REG_SR_BIDIR(f32, any, f32, gOIhw4i16o4i),
 
@@ -267,6 +269,7 @@ static const impl_list_map_t regular_impl_list_map {
         REG_SR_BIDIR(f32, any, f32, gOdhwi16o),
         REG_SR_BIDIR(f32, any, f32, gOIdhw16o16i),
         REG_SR_BIDIR(f32, any, f32, gOIdhw16i16o),
+        REG_SR_BIDIR(f32, any, f32, gIOdhw16o16i),
 
         REG_SR(f32, any, f32, any, fmt_order::any, spec::reference),
 
@@ -325,6 +328,15 @@ static const impl_list_map_t regular_impl_list_map {
 
         jit_uni_reorder_create,
 
+        REG_SR_BIDIR(bf16, any, f32, nChw16c),
+        REG_SR_BIDIR(bf16, any, f32, nCdhw16c),
+
+        REG_SR_BIDIR(bf16, any, bf16, nChw16c),
+        REG_SR_BIDIR(bf16, any, bf16, nCdhw16c),
+
+        REG_SR_BIDIR(bf16, any, f32, OIdhw16o16i),
+        REG_SR_BIDIR(bf16, any, f32, OIdhw16i16o),
+
         REG_SR(bf16, any, bf16, any, fmt_order::any, spec::reference),
         REG_SR(bf16, any, f32, any, fmt_order::any, spec::reference),
 
@@ -363,6 +375,8 @@ static const impl_list_map_t regular_impl_list_map {
 
     // s8 ->
     {{s8, data_type::undef, 0}, {
+       rnn_weights_reorder_s8_t<s8>::pd_t::create,
+
        REG_FAST_DIRECT_COPY_COMMA(s8, f32)
        REG_FAST_DIRECT_COPY_COMMA(s8, s32)
        REG_FAST_DIRECT_COPY_COMMA(s8, s8)
@@ -417,17 +431,25 @@ static const impl_list_map_t comp_s8s8_impl_list_map {
     /*
     // f32 -> s8
     {{f32, s8, 3}, {
+        REG_SR(f32, any, s8, wio, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, oiw, s8, OIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wio, s8, OIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, oiw, s8, OIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wio, s8, OIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, oiw, s8, OIw4o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wio, s8, OIw4o4i, fmt_order::keep, spec::conv_s8s8),
 
         nullptr,
     }},
     {{f32, s8, 4}, {
         REG_SR(f32, any, s8, hwio, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, any, s8, wigo, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, goiw, s8, gOIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wigo, s8, gOIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, goiw, s8, gOIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wigo, s8, gOIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, goiw, s8, gOIw4o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wigo, s8, gOIw4o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, oihw, s8, OIhw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, hwio, s8, OIhw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, hwio, s8, OIhw2i8o4i, fmt_order::keep, spec::conv_s8s8),
@@ -435,7 +457,9 @@ static const impl_list_map_t comp_s8s8_impl_list_map {
         REG_SR(f32, hwio, s8, OIhw4o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, oihw, s8, OIhw4o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, goiw, s8, Goiw16g, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wigo, s8, Goiw16g, fmt_order::keep, spec::conv_s8s8),
         REG_SR(f32, goiw, s8, Goiw8g, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(f32, wigo, s8, Goiw8g, fmt_order::keep, spec::conv_s8s8),
 
         nullptr,
     }},
@@ -471,17 +495,25 @@ static const impl_list_map_t comp_s8s8_impl_list_map {
     }},
     // s8 -> s8
     {{s8, s8, 3}, {
+        REG_SR(s8, any, s8, wio, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, oiw, s8, OIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wio, s8, OIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, oiw, s8, OIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wio, s8, OIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, oiw, s8, OIw4o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wio, s8, OIw4o4i, fmt_order::keep, spec::conv_s8s8),
 
         nullptr,
     }},
     {{s8, s8, 4}, {
         REG_SR(s8, any, s8, hwio, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, any, s8, wigo, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, goiw, s8, gOIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wigo, s8, gOIw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, goiw, s8, gOIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wigo, s8, gOIw2i8o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, goiw, s8, gOIw4o4i, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wigo, s8, gOIw4o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, hwio, s8, OIhw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, oihw, s8, OIhw4i16o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, hwio, s8, OIhw2i8o4i, fmt_order::keep, spec::conv_s8s8),
@@ -489,7 +521,9 @@ static const impl_list_map_t comp_s8s8_impl_list_map {
         REG_SR(s8, hwio, s8, OIhw4o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, oihw, s8, OIhw4o4i, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, goiw, s8, Goiw16g, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wigo, s8, Goiw16g, fmt_order::keep, spec::conv_s8s8),
         REG_SR(s8, goiw, s8, Goiw8g, fmt_order::keep, spec::conv_s8s8),
+        REG_SR(s8, wigo, s8, Goiw8g, fmt_order::keep, spec::conv_s8s8),
 
         nullptr,
     }},
