@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2019 Intel Corporation
+* Copyright 2017-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -1757,17 +1757,17 @@ const xbyak_gemm *get_xbyak_gemm(
     return kernel_table[isTransA][isTransB][hasBias][beta_idx(beta)];
 }
 
-void sgemm_nocopy_driver(const char *transa, const char *transb, int m, int n,
-        int k, const float *alpha, const float *a, dim_t lda, const float *b,
-        dim_t ldb, const float *beta, float *c, dim_t ldc, const float *bias,
-        float *ws) {
+void sgemm_nocopy_driver(const char *transa, const char *transb, dim_t m,
+        dim_t n, dim_t k, const float *alpha, const float *a, dim_t lda,
+        const float *b, dim_t ldb, const float *beta, float *c, dim_t ldc,
+        const float *bias, float *ws) {
 
     bool isTransA = (*transa == 'T' || *transa == 't');
     bool isTransB = (*transb == 'T' || *transb == 't');
 
-    int Bm, sizeM, Bn, sizeN, Bk, sizeK;
+    dim_t Bm, sizeM, Bn, sizeN, Bk, sizeK;
 
-    int i, j;
+    dim_t i, j;
 
     if ((m <= 0) || (n <= 0)) return;
 
@@ -1795,7 +1795,7 @@ void sgemm_nocopy_driver(const char *transa, const char *transb, int m, int n,
     auto ker_b0 = get_xbyak_gemm(isTransA, isTransB, 0.0, false);
     assert(ker_bn && ker_b1 && ker_b0);
 
-    int BM = 4032, BN, BK;
+    dim_t BM = 4032, BN, BK;
     if (mayiuse(avx512_core)) {
         BN = isTransA ? 384 : 64;
         BK = 384;
@@ -1851,16 +1851,14 @@ void sgemm_nocopy_driver(const char *transa, const char *transb, int m, int n,
                 }
                 if (Bk == 0) {
                     if (*beta == 0.0 && bias == nullptr)
-                        (*ker_b0)((dim_t)sizeM, (dim_t)sizeN, (dim_t)sizeK,
-                                alpha, curA, lda, curB, ldb, beta, curC, ldc,
-                                curBias, ws);
+                        (*ker_b0)(sizeM, sizeN, sizeK, alpha, curA, lda, curB,
+                                ldb, beta, curC, ldc, curBias, ws);
                     else
-                        (*ker_bn)((dim_t)sizeM, (dim_t)sizeN, (dim_t)sizeK,
-                                alpha, curA, lda, curB, ldb, beta, curC, ldc,
-                                curBias, ws);
+                        (*ker_bn)(sizeM, sizeN, sizeK, alpha, curA, lda, curB,
+                                ldb, beta, curC, ldc, curBias, ws);
                 } else {
-                    (*ker_b1)((dim_t)sizeM, (dim_t)sizeN, (dim_t)sizeK, alpha,
-                            curA, lda, curB, ldb, beta, curC, ldc, curBias, ws);
+                    (*ker_b1)(sizeM, sizeN, sizeK, alpha, curA, lda, curB, ldb,
+                            beta, curC, ldc, curBias, ws);
                 }
             }
         }
@@ -1871,9 +1869,10 @@ void sgemm_nocopy_driver(const char *transa, const char *transb, int m, int n,
 } // namespace avx512_common_gemm_f32
 
 dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
-        const int *p_m, const int *p_n, const int *p_k, const float *p_alpha,
-        const float *A, const int *p_lda, const float *B, const int *p_ldb,
-        const float *p_beta, float *C, const int *p_ldc, const float *bias) {
+        const dim_t *p_m, const dim_t *p_n, const dim_t *p_k,
+        const float *p_alpha, const float *A, const dim_t *p_lda,
+        const float *B, const dim_t *p_ldb, const float *p_beta, float *C,
+        const dim_t *p_ldc, const float *bias) {
 
     using namespace dnnl::impl::utils;
     using namespace avx512_common_gemm_f32;
@@ -1883,26 +1882,27 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
         return ref_gemm(transa, transb, p_m, p_n, p_k, p_alpha, A, p_lda, B,
                 p_lda, p_beta, C, p_ldc, bias);
 
-    int nthr = (dnnl_in_parallel()) ? 1 : dnnl_get_max_threads();
+    int nthr_to_use = (dnnl_in_parallel()) ? 1 : dnnl_get_max_threads();
 
-    int m = *p_m;
-    int n = *p_n;
-    int k = *p_k;
+    dim_t m = *p_m;
+    dim_t n = *p_n;
+    dim_t k = *p_k;
     dim_t lda = *p_lda;
     dim_t ldb = *p_ldb;
     dim_t ldc = *p_ldc;
     float beta = *p_beta;
-    int MB, NB, KB;
+    dim_t MB, NB, KB;
 
     int nthr_m, nthr_n, nthr_k, nthr_mn;
 
     // Determine threading partitioning
     calc_nthr_nocopy_avx512_common(
-            m, n, k, nthr, &nthr_m, &nthr_n, &nthr_k, &MB, &NB, &KB);
+            m, n, k, nthr_to_use, &nthr_m, &nthr_n, &nthr_k, &MB, &NB, &KB);
     assert(IMPLICATION(!dnnl_thr_syncable(), nthr_k == 1));
 
     // May not happen, but just in case
-    if (nthr < nthr_m * nthr_n * nthr_k) nthr = nthr_m * nthr_n * nthr_k;
+    if (nthr_to_use < nthr_m * nthr_n * nthr_k)
+        nthr_to_use = nthr_m * nthr_n * nthr_k;
 
     nthr_mn = nthr_m * nthr_n;
 
@@ -1914,32 +1914,44 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
 
     if (nthr_k > 1) {
         ompstatus_ = (unsigned char *)malloc(
-                nthr * CACHE_LINE_SIZE, CACHE_LINE_SIZE);
+                nthr_to_use * CACHE_LINE_SIZE, CACHE_LINE_SIZE);
         if (!ompstatus_) return dnnl_out_of_memory;
 
         ompstatus = (unsigned char volatile *)ompstatus_;
         assert(ompstatus);
 
-        for (int i = 0; i < nthr; i++)
+        for (int i = 0; i < nthr_to_use; i++)
             ompstatus[i * CACHE_LINE_SIZE] = 0;
 
         c_buffers = (float *)malloc(
-                nthr_m * nthr_n * (nthr_k - 1) * MB * NB * sizeof(float),
+                sizeof(*c_buffers) * nthr_m * nthr_n * (nthr_k - 1) * MB * NB,
                 PAGE_4K);
+        if (!c_buffers) {
+            free(ompstatus_);
+            return dnnl_out_of_memory;
+        }
     }
 
     const size_t ws_elems_per_thr = (size_t)k * 48 + 64;
     const size_t ws_size_per_thr
             = rnd_up(ws_elems_per_thr * sizeof(float), PAGE_4K);
     if (k > STACK_K_CAPACITY) {
-        ws_buffers = (float *)malloc(nthr * ws_size_per_thr, PAGE_4K);
+        ws_buffers = (float *)malloc(nthr_to_use * ws_size_per_thr, PAGE_4K);
+        if (!ws_buffers) {
+            free(ompstatus_);
+            free(c_buffers);
+            return dnnl_out_of_memory;
+        }
     }
 
-    parallel_nd(nthr, [&](const int ithr) {
+    parallel(nthr_to_use, [&](int ithr, int nthr) {
+        assert(nthr == nthr_to_use);
+        MAYBE_UNUSED(nthr);
+
         int ithr_m, ithr_n, ithr_k, ithr_mn;
-        int m_from, m_to, myM;
-        int n_from, n_to, myN;
-        int k_from, k_to, myK;
+        dim_t m_from, m_to, myM;
+        dim_t n_from, n_to, myN;
+        dim_t k_from, k_to, myK;
         int cbase, ibase;
         const float *myA, *myB, *myBias = nullptr;
         float *myC = C, myBeta;
@@ -1948,7 +1960,7 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
                 : 0;
         dim_t ld = ldc;
 
-        int sum_later = (dnnl_get_num_threads() < nthr_m * nthr_n * nthr_k);
+        int sum_later = nthr < nthr_m * nthr_n * nthr_k;
 
         if (ithr < nthr_m * nthr_n * nthr_k) {
 
@@ -1999,7 +2011,7 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
                     ld = ldc;
                     if (bias) myBias = &(bias[m_from]);
                 } else {
-                    myC = c_buffers + (dim_t)MB * NB * (cbase + ithr_k - 1);
+                    myC = c_buffers + MB * NB * (cbase + ithr_k - 1);
                     myBeta = 0.0;
                     ld = MB;
                     myBias = nullptr;
@@ -2015,14 +2027,13 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
             if (nthr_k > 1 && !sum_later) {
 
                 // sum matrices partitioned along K dimension
-                int n1, n2;
+                dim_t n1, n2;
 
                 partition_unit_diff(ithr_k, nthr_k, myN, &n1, &n2);
 
                 if (ithr_k > 0) {
 
-                    myC = c_buffers + (dim_t)MB * NB * (cbase + ithr_k - 1)
-                            + (dim_t)n1 * MB;
+                    myC = c_buffers + MB * NB * (cbase + ithr_k - 1) + n1 * MB;
                     /* need to wait until main thread finishes */
                     while (ompstatus[ibase * CACHE_LINE_SIZE] != 1) {};
 
@@ -2034,8 +2045,7 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
                 for (int ik = 1; ik < nthr_k; ++ik) {
                     if (ik != ithr_k) {
 
-                        myC = c_buffers + (dim_t)MB * NB * (cbase + ik - 1)
-                                + (dim_t)n1 * MB;
+                        myC = c_buffers + MB * NB * (cbase + ik - 1) + n1 * MB;
 
                         while (ompstatus[(ibase + ik) * CACHE_LINE_SIZE] != 1) {
                         };
@@ -2051,10 +2061,13 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
     // handle C summation later
     if (nthr_k > 1 && ompstatus[0] == 0) {
 
-        parallel_nd(nthr, [&](const int ithr) {
+        parallel(nthr_to_use, [&](int ithr, int nthr) {
+            assert(nthr == nthr_to_use);
+            MAYBE_UNUSED(nthr);
+
             int ithr_m, ithr_n, ithr_k, ithr_mn;
-            int m_from, m_to, myM;
-            int n_from, n_to, myN;
+            dim_t m_from, m_to, myM;
+            dim_t n_from, n_to, myN;
             int cbase;
             float *myC = C;
 
@@ -2085,14 +2098,14 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
 
                 if (nthr_k > 1) {
                     // sum matrices partitioned along K dimension
-                    int n1, n2;
+                    dim_t n1, n2;
 
                     partition_unit_diff(ithr_k, nthr_k, myN, &n1, &n2);
 
                     if (ithr_k > 0) {
 
-                        myC = c_buffers + (dim_t)MB * NB * (cbase + ithr_k - 1)
-                                + (dim_t)n1 * MB;
+                        myC = c_buffers + MB * NB * (cbase + ithr_k - 1)
+                                + n1 * MB;
 
                         /* my cache is hot */
                         sum_two_matrices(myM, n2, myC, MB,
@@ -2102,8 +2115,8 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
                     for (int ik = 1; ik < nthr_k; ++ik) {
                         if (ik != ithr_k) {
 
-                            myC = c_buffers + (dim_t)MB * NB * (cbase + ik - 1)
-                                    + (dim_t)n1 * MB;
+                            myC = c_buffers + MB * NB * (cbase + ik - 1)
+                                    + n1 * MB;
 
                             sum_two_matrices(myM, n2, myC, MB,
                                     &C[m_from + (n_from + n1) * ldc], ldc);
