@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2019 Intel Corporation
+* Copyright 2016-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -127,10 +127,6 @@ template void im2col_3d(
 template void im2col_3d(const jit_gemm_conv_conf_t &jcp, const bfloat16_t *im,
         bfloat16_t *col, int od);
 
-inline int saturate(int low, int upper, int value) {
-    return nstl::max(low, nstl::min(upper, value));
-}
-
 /* imtr[ic][od][oh][ow] <-- im[id][ih][iw][ic]*/
 template <typename T>
 void transpose_u8(const jit_gemm_conv_conf_t &jcp, const T *__restrict im,
@@ -814,7 +810,7 @@ status_t init_conf(jit_gemm_conv_conf_t &jcp,
     bool is_depthwise = jcp.ic == 1 && jcp.oc == 1 && jcp.ngroups != 1;
 
     // TODO: maybe mitigate blocking restriction
-    const int L2 = get_cache_size(2, true) / data_size;
+    const int L2 = get_per_core_cache_size(2) / data_size;
     const int gemm_thrld = 64 * 1024;
 
     if (is_int8_conv) {
