@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2019 Intel Corporation
+* Copyright 2017-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -106,7 +106,7 @@ static int prepare_fwd_no_stats(const prb_t *p, dnn_mem_t &src, dnn_mem_t &mean,
             : 1.f;
     assert((exact_bits - ceilf(log2f(L * density))) / 2 >= flex_bits);
 
-    print(6, "check_alg: %s, density = %g, flex_bits = " IFMT "\n",
+    BENCHDNN_PRINT(6, "check_alg: %s, density = %g, flex_bits = " IFMT "\n",
             check_alg2str(alg), density, flex_bits);
 
     dnnl::impl::parallel_nd(p->ic, [&](int64_t c) {
@@ -216,7 +216,8 @@ static int prepare_bwd(const prb_t *p, dnn_mem_t &src, dnn_mem_t &d_dst,
     const float density
             = MIN3(1.f, (1 << exact_bits) / ub_dg, (1 << exact_bits) / ub_db);
 
-    print(5, "prep_bwd: k:" IFMT ", P:" IFMT " log2P:" IFMT ", density = %g\n",
+    BENCHDNN_PRINT(5,
+            "prep_bwd: k:" IFMT ", P:" IFMT " log2P:" IFMT ", density = %g\n",
             k, P, log2P, density);
 
     dnnl::impl::parallel_nd(p->ic, [&](int64_t c) {
@@ -397,7 +398,8 @@ static int compare(const prb_t *p, data_kind_t kind, const dnn_mem_t &fp_mem,
             }
 
             std::string ind_str = ss.str();
-            print(0, "[%4ld][%s%s][%s] fp:%8g dt:%8g diff:%8g rdiff:%8g\n",
+            BENCHDNN_PRINT(0,
+                    "[%4ld][%s%s][%s] fp:%8g dt:%8g diff:%8g rdiff:%8g\n",
                     (long)i, p->dir & FLAG_BWD ? "D_" : "", skind,
                     ind_str.c_str(), fp, dt, diff, rel_diff);
         }
@@ -413,7 +415,7 @@ static int compare(const prb_t *p, data_kind_t kind, const dnn_mem_t &fp_mem,
 
     if (r->errors || verbose >= 5) {
         const int vl = r->errors ? 0 : 2;
-        print(vl,
+        BENCHDNN_PRINT(vl,
                 "@@@ [%s%s] diff: l0(``%g``) "
                 "l1:(%g,%g,%g,``%g``) "
                 "l2:(%g,%g,%g,``%g``) "
@@ -462,7 +464,7 @@ int check_fwd_ws(const dnn_mem_t &dst_dt, const dnn_mem_t &ws_dt, res_t *r) {
             bool dump = false || (!ok && (r->errors < 10 || verbose >= 10))
                     || (verbose >= 50 && i < 30);
             if (dump) {
-                print(0, "[%4ld] ws exp:%d got:%d (data:%g:%a)\n",
+                BENCHDNN_PRINT(0, "[%4ld] ws exp:%d got:%d (data:%g:%a)\n",
                         (long)(i + j), want, bit_set, data, data);
             }
 
@@ -483,8 +485,8 @@ int check_fwd_ws(const dnn_mem_t &dst_dt, const dnn_mem_t &ws_dt, res_t *r) {
     return r->state == FAILED ? FAIL : OK;
 }
 
-static int init_pd(const prb_t *p, dnnl_batch_normalization_desc_t &bd,
-        dnnl_primitive_desc_t &bpd, res_t *r) {
+static int init_pd(const prb_t *p, dnnl_primitive_desc_t &bpd, res_t *r) {
+    dnnl_batch_normalization_desc_t bd;
     dnnl_memory_desc_t data_d;
 
     dnnl_dims_t data_dims_0d = {p->mb, p->ic};
@@ -497,8 +499,8 @@ static int init_pd(const prb_t *p, dnnl_batch_normalization_desc_t &bd,
             : p->ndims == 4 ? data_dims_2d
                             : p->ndims == 3 ? data_dims_1d : data_dims_0d;
 
-    DNN_SAFE(dnnl_memory_desc_init_by_tag(
-                     &data_d, p->ndims, data_dims, p->dt, p->tag),
+    DNN_SAFE(dnnl_memory_desc_init_by_tag(&data_d, p->ndims, data_dims, p->dt,
+                     convert_tag(p->tag, p->ndims)),
             WARN);
 
     auto flags = (dnnl_normalization_flags_t)p->flags;
@@ -533,7 +535,9 @@ static int init_pd(const prb_t *p, dnnl_batch_normalization_desc_t &bd,
         else
             SAFE(init_fwd_status, WARN);
     }
-    auto dnnl_attr = create_dnnl_attr(p->attr, 1, NULL);
+
+    auto dnnl_attr = create_dnnl_attr(p->attr);
+
     dnnl_status_t init_status = dnnl_primitive_desc_create(
             &bpd, &bd, dnnl_attr, engine_tgt, hint_fwd_pd);
 
@@ -546,18 +550,18 @@ static int init_pd(const prb_t *p, dnnl_batch_normalization_desc_t &bd,
         SAFE(init_status, WARN);
 
     const char *impl_str = query_impl_info(bpd);
-    if (maybe_skip(skip_impl, impl_str)) {
-        print(2, "SKIPPED: dnnl implementation: %s\n", impl_str);
+    if (maybe_skip(impl_str)) {
+        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n", impl_str);
         DNN_SAFE(dnnl_primitive_desc_destroy(bpd), WARN);
         return r->state = SKIPPED, OK;
     } else {
-        print(5, "dnnl implementation: %s\n", impl_str);
+        BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
         if (!strstr(impl_str, "jit")) {
-            print(2, "WARNING: %s",
+            BENCHDNN_PRINT(2, "WARNING: %s",
                     "accuracy of the implementation being tested "
                     "depends on the compiler and might give "
                     "false-positives.\n");
-            print(2, "         %s",
+            BENCHDNN_PRINT(2, "         %s",
                     "please consider recompiling the sources with"
                     " `-prec-div -fp-model precise` for a reliable testing.\n");
         }
@@ -579,7 +583,8 @@ static int cvt_mask_to_ws(
             : p->ndims == 4 ? data_dims_2d
                             : p->ndims == 3 ? data_dims_1d : data_dims_0d;
 
-    dnn_mem_t data(p->ndims, data_dims, dnnl_f32, p->tag, engine_tgt);
+    dnn_mem_t data(p->ndims, data_dims, dnnl_f32, convert_tag(p->tag, p->ndims),
+            engine_tgt);
     SAFE(data.reorder(mask_fp), WARN);
 
     dnn_mem_t mean(1, &p->ic, dnnl_f32, dnnl_x, engine_tgt);
@@ -620,48 +625,64 @@ static int cvt_mask_to_ws(
 int doit(const prb_t *p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
 
-    dnnl_batch_normalization_desc_t bd;
     dnnl_primitive_desc_t bpd;
-    dnnl_primitive_t b;
-
-    SAFE(init_pd(p, bd, bpd, r), WARN);
+    SAFE(init_pd(p, bpd, r), WARN);
     if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
 
-    const auto fp = dnnl_f32;
-    const auto tag = get_default_tag(bd.data_desc.ndims);
-    const auto &data_desc = bd.data_desc;
+    dnnl_primitive_t b;
+    DNN_SAFE(dnnl_primitive_create(&b, bpd), WARN);
+    DNN_SAFE(dnnl_primitive_desc_destroy(bpd), CRIT);
 
-    dnn_mem_t src_fp(data_desc, fp, tag, engine_tgt);
-    dnn_mem_t src_dt(data_desc, engine_tgt);
+    const_dnnl_primitive_desc_t const_pd;
+    DNN_SAFE(dnnl_primitive_get_primitive_desc(b, &const_pd), CRIT);
+
+    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
+        DNN_SAFE_V(dnnl_primitive_destroy(b));
+        return r->state = SKIPPED, OK;
+    }
+
+    const auto q = [&](int index = 0) -> const dnnl_memory_desc_t & {
+        return *dnnl_primitive_desc_query_md(
+                const_pd, dnnl_query_exec_arg_md, index);
+    };
+
+    const auto &data_md = q(DNNL_ARG_SRC);
+    const auto &mean_md = q(DNNL_ARG_MEAN);
+    const auto &var_md = q(DNNL_ARG_VARIANCE);
+    const auto &ss_md = q(DNNL_ARG_SCALE_SHIFT);
+    const auto &ws_md = q(DNNL_ARG_WORKSPACE);
+    const auto &scratchpad_md = q(DNNL_ARG_SCRATCHPAD);
+
+    const auto fp = dnnl_f32;
+    const auto tag = get_abx_tag(p->ndims);
+
+    dnn_mem_t src_fp(data_md, fp, tag, engine_tgt);
+    dnn_mem_t src_dt(data_md, engine_tgt);
 
     dnn_mem_t &dst_fp = src_fp; // in-place in ref code
     dnn_mem_t placeholder_dst_dt;
-    if (!p->inplace) { placeholder_dst_dt = dnn_mem_t(data_desc, engine_tgt); }
+    if (!p->inplace) { placeholder_dst_dt = dnn_mem_t(data_md, engine_tgt); }
     dnn_mem_t &dst_dt = p->inplace ? src_dt : placeholder_dst_dt;
 
+    // On inference w/o global stats the batch norm doesn't require stat
+    // memories. Hence, we need to prepare the mean_fp and var_fp ourselves.
     const dnnl_dims_t dims1d = {p->ic};
-    dnn_mem_t mean_fp(1, dims1d, fp, dnnl_x, engine_tgt),
-            mean_dt(mean_fp.md_, engine_tgt);
-    dnn_mem_t var_fp(1, dims1d, fp, dnnl_x, engine_tgt),
-            var_dt(var_fp.md_, engine_tgt);
+    dnn_mem_t mean_fp(1, dims1d, fp, get_abx_tag(1), engine_tgt);
+    dnn_mem_t mean_dt(mean_md, engine_tgt);
+    dnn_mem_t var_fp(1, dims1d, fp, get_abx_tag(1), engine_tgt);
+    dnn_mem_t var_dt(var_md, engine_tgt);
 
-    const dnnl_dims_t dims2d = {2, p->ic};
-    dnn_mem_t ss_fp(2, dims2d, fp, dnnl_nc, engine_tgt),
-            ss_dt(ss_fp.md_, engine_tgt);
-    dnn_mem_t d_ss_fp(2, dims2d, fp, dnnl_nc, engine_tgt),
-            d_ss_dt(d_ss_fp.md_, engine_tgt);
+    dnn_mem_t ss_fp(ss_md, fp, get_abx_tag(ss_md.ndims), engine_tgt);
+    dnn_mem_t ss_dt(ss_md, engine_tgt);
+    dnn_mem_t d_ss_fp(ss_md, fp, get_abx_tag(ss_md.ndims), engine_tgt);
+    dnn_mem_t d_ss_dt(ss_md, engine_tgt);
 
+    if ((p->flags & FUSE_NORM_RELU) && !(p->dir & FLAG_INF))
+        SAFE(ws_md.ndims != 0 ? OK : FAIL, WARN);
     dnn_mem_t ws_fp(src_fp.md_, engine_tgt);
-    dnn_mem_t ws_dt;
-    if ((p->flags & FUSE_NORM_RELU) && !(p->dir & FLAG_INF)) {
-        const auto ws_md
-                = dnnl_primitive_desc_query_md(bpd, dnnl_query_workspace_md, 0);
-        SAFE(ws_md != NULL ? OK : FAIL, WARN);
-        ws_dt = dnn_mem_t(*ws_md, engine_tgt);
-    }
+    dnn_mem_t ws_dt(ws_md, engine_tgt);
 
-    DNN_SAFE(dnnl_primitive_create(&b, bpd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(bpd), CRIT);
+    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
 
     dnn_mem_t d_dst_dt, placeholder_d_src_dt;
 
@@ -669,14 +690,14 @@ int doit(const prb_t *p, res_t *r) {
 
     if (p->dir & FLAG_FWD) {
         if (prepare_fwd(p, src_fp, mean_fp, var_fp, ss_fp) != OK) {
-            r->state = MISTRUSTED;
-            goto out;
+            DNN_SAFE_V(dnnl_primitive_destroy(b));
+            return r->state = MISTRUSTED, OK;
         }
 
         SAFE(src_dt.reorder(src_fp), WARN);
 
         args.set(DNNL_ARG_SRC, src_dt);
-        args.set(DNNL_ARG_DST, p->inplace ? src_dt : dst_dt);
+        args.set(DNNL_ARG_DST, dst_dt);
 
         if (p->flags & GLOB_STATS) {
             /* prepare mean & var if they are inputs */
@@ -686,12 +707,10 @@ int doit(const prb_t *p, res_t *r) {
         args.set(DNNL_ARG_MEAN, mean_dt);
         args.set(DNNL_ARG_VARIANCE, var_dt);
 
-        if (p->flags & USE_SCALESHIFT) {
-            SAFE(ss_dt.reorder(ss_fp), WARN);
-            args.set(DNNL_ARG_SCALE_SHIFT, ss_dt);
-        }
-
-        if (p->flags & FUSE_NORM_RELU) args.set(DNNL_ARG_WORKSPACE, ws_dt);
+        if (p->flags & USE_SCALESHIFT) { SAFE(ss_dt.reorder(ss_fp), WARN); }
+        args.set(DNNL_ARG_SCALE_SHIFT, ss_dt);
+        args.set(DNNL_ARG_WORKSPACE, ws_dt);
+        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
         DNN_SAFE(execute_and_wait(b, stream_tgt, args), WARN);
 
@@ -699,7 +718,6 @@ int doit(const prb_t *p, res_t *r) {
             compute_ref_fwd(p, src_fp, mean_fp, var_fp, ss_fp, dst_fp);
             if (!(p->flags & GLOB_STATS) && !(p->dir & FLAG_INF)) {
                 SAFE(compare(p, MEAN, mean_fp, mean_dt, r), WARN);
-
                 SAFE(compare(p, VAR, var_fp, var_dt, r), WARN);
             }
             dnn_mem_t dst(dst_dt, fp, tag, engine_tgt);
@@ -709,31 +727,27 @@ int doit(const prb_t *p, res_t *r) {
             }
         }
     } else {
-        const_dnnl_primitive_desc_t const_bpd;
-        DNN_SAFE(dnnl_primitive_get_primitive_desc(b, &const_bpd), CRIT);
-        const auto &d_data_desc = *dnnl_primitive_desc_query_md(
-                const_bpd, dnnl_query_diff_src_md, 0);
+        const auto &d_data_md = q(DNNL_ARG_DIFF_DST);
 
-        dnn_mem_t d_dst_fp(d_data_desc, fp, tag, engine_tgt);
-        d_dst_dt = dnn_mem_t(d_data_desc, engine_tgt);
+        dnn_mem_t d_dst_fp(d_data_md, fp, tag, engine_tgt);
+        d_dst_dt = dnn_mem_t(d_data_md, engine_tgt);
 
         dnn_mem_t &d_src_fp = d_dst_fp; // in-place in ref code
         if (!p->inplace) {
-            placeholder_d_src_dt = dnn_mem_t(d_data_desc, engine_tgt);
+            placeholder_d_src_dt = dnn_mem_t(d_data_md, engine_tgt);
         }
         dnn_mem_t &d_src_dt = p->inplace ? d_dst_dt : placeholder_d_src_dt;
 
         if (prepare_bwd(p, src_fp, d_dst_fp, mean_fp, var_fp, ss_fp, ws_fp)
                 != OK) {
-            r->state = MISTRUSTED;
-            goto out;
+            DNN_SAFE_V(dnnl_primitive_destroy(b));
+            return r->state = MISTRUSTED, OK;
         }
 
         SAFE(src_dt.reorder(src_fp), WARN);
         SAFE(d_dst_dt.reorder(d_dst_fp), WARN);
 
         args.set(DNNL_ARG_SRC, src_dt);
-
         args.set(DNNL_ARG_DIFF_DST, d_dst_dt);
         args.set(DNNL_ARG_DIFF_SRC, d_src_dt);
 
@@ -742,16 +756,15 @@ int doit(const prb_t *p, res_t *r) {
         args.set(DNNL_ARG_MEAN, mean_dt);
         args.set(DNNL_ARG_VARIANCE, var_dt);
 
-        if (p->flags & USE_SCALESHIFT) {
-            SAFE(ss_dt.reorder(ss_fp), WARN);
-            args.set(DNNL_ARG_SCALE_SHIFT, ss_dt);
-            args.set(DNNL_ARG_DIFF_SCALE_SHIFT, d_ss_dt);
-        }
+        if (p->flags & USE_SCALESHIFT) { SAFE(ss_dt.reorder(ss_fp), WARN); }
+        args.set(DNNL_ARG_SCALE_SHIFT, ss_dt);
+        args.set(DNNL_ARG_DIFF_SCALE_SHIFT, d_ss_dt);
 
         if (p->flags & FUSE_NORM_RELU) {
             SAFE(cvt_mask_to_ws(p, ws_fp, ws_dt), WARN);
-            args.set(DNNL_ARG_WORKSPACE, ws_dt);
         }
+        args.set(DNNL_ARG_WORKSPACE, ws_dt);
+        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
         DNN_SAFE(execute_and_wait(b, stream_tgt, args), WARN);
 
@@ -768,8 +781,8 @@ int doit(const prb_t *p, res_t *r) {
 
     measure_perf(r->timer, b, args);
 
-out:
-    DNN_SAFE(dnnl_primitive_destroy(b), CRIT);
+    DNN_SAFE_V(dnnl_primitive_destroy(b));
+
     return OK;
 }
 
