@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2019 Intel Corporation
+* Copyright 2016-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -60,7 +60,11 @@ using dnnl::impl::f16_support::float16_t;
 using memory = dnnl::memory;
 
 bool is_current_test_failed();
+
+#ifdef DNNL_TEST_WITH_ENGINE_PARAM
 dnnl::engine::kind get_test_engine_kind();
+dnnl::engine get_test_engine();
+#endif
 
 template <typename data_t>
 struct data_traits {};
@@ -469,7 +473,7 @@ struct test_convolution_eltwise_params_t {
 
 template <typename F>
 bool catch_expected_failures(const F &f, bool expect_to_fail,
-        dnnl_status_t expected_status, bool ignore_unimplemented = true) {
+        dnnl_status_t expected_status, bool ignore_unimplemented = false) {
     try {
         f();
     } catch (const dnnl::error &e) {
@@ -580,6 +584,7 @@ struct allows_attr_t {
     bool po_sum;
     bool po_eltwise;
     bool zp;
+    bool scales;
 };
 
 using engine = dnnl::engine;
@@ -587,48 +592,44 @@ using engine = dnnl::engine;
 template <typename op_desc_t, typename pd_t>
 void test_fwd_pd_attr(const op_desc_t &op_desc, const engine &eng) {
     dnnl::primitive_attr attr;
-    pd_t new_pd {};
-    EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr, eng));
+    EXPECT_NO_THROW(pd_t pd(op_desc, attr, eng));
 }
 
 template <typename op_desc_t, typename pd_t>
 void test_fwd_pd_attr_oscale(
         const op_desc_t &op_desc, const engine &eng, bool supports_oscale) {
     dnnl::primitive_attr attr_oscale;
-    attr_oscale.set_output_scales(0, {2.});
-    pd_t new_pd {};
+    attr_oscale.set_output_scales(0, {2.f});
     if (supports_oscale)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_oscale, eng));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_oscale, eng));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_oscale, eng));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_oscale, eng));
 }
 
 template <typename op_desc_t, typename pd_t>
 void test_fwd_pd_attr_po_sum(
         const op_desc_t &op_desc, const engine &eng, bool supports_po_sum) {
-    dnnl::primitive_attr attr_po_sum;
     dnnl::post_ops ops_sum;
     ops_sum.append_sum(1.1f);
+    dnnl::primitive_attr attr_po_sum;
     attr_po_sum.set_post_ops(ops_sum);
-    pd_t new_pd {};
     if (supports_po_sum)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_po_sum, eng));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_sum, eng));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_po_sum, eng));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_sum, eng));
 }
 
 template <typename op_desc_t, typename pd_t>
 void test_fwd_pd_attr_po_eltwise(
         const op_desc_t &op_desc, const engine &eng, bool supports_po_eltwise) {
-    dnnl::primitive_attr attr_po_eltwise;
     dnnl::post_ops ops_eltwise;
     ops_eltwise.append_eltwise(1.0f, dnnl::algorithm::eltwise_relu, 0.f, 0.f);
+    dnnl::primitive_attr attr_po_eltwise;
     attr_po_eltwise.set_post_ops(ops_eltwise);
-    pd_t new_pd {};
     if (supports_po_eltwise)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_po_eltwise, eng));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_eltwise, eng));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_po_eltwise, eng));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_eltwise, eng));
 }
 
 template <typename op_desc_t, typename pd_t>
@@ -636,11 +637,39 @@ void test_fwd_pd_attr_zp(
         const op_desc_t &op_desc, const engine &eng, bool supports_zero_point) {
     dnnl::primitive_attr attr_zp;
     attr_zp.set_zero_points(DNNL_ARG_SRC, 0, {1});
-    pd_t new_pd {};
     if (supports_zero_point)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_zp, eng));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_zp, eng));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_zp, eng));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_zp, eng));
+}
+
+template <typename op_desc_t, typename pd_t>
+void test_fwd_pd_attr_scales(
+        const op_desc_t &op_desc, const engine &eng, bool supports_scales) {
+    using dt_t = memory::data_type;
+
+    dnnl::primitive_attr attr_scales;
+    attr_scales.set_scales(DNNL_ARG_SRC, 0, {2.f});
+
+    pd_t new_pd(op_desc, eng);
+    const auto dt = new_pd.src_desc().data.data_type;
+    const bool dt_is_integral
+            = dt == dt_t::s32 || dt == dt_t::s8 || dt == dt_t::u8;
+
+    // Scales are supposed to work on int8 data type only.
+    if (supports_scales && dt_is_integral) {
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_scales, eng));
+
+        // Check oscale and scales don't work together
+        dnnl::primitive_attr attr_oscale_scales;
+        attr_oscale_scales.set_output_scales(0, {2.f});
+        EXPECT_ANY_THROW(attr_oscale_scales.set_scales(DNNL_ARG_SRC, 0, {2.f}));
+
+        dnnl::primitive_attr attr_scales_oscale;
+        attr_scales_oscale.set_scales(DNNL_ARG_SRC, 0, {2.f});
+        EXPECT_ANY_THROW(attr_scales_oscale.set_output_scales(0, {2.f}));
+    } else
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_scales, eng));
 }
 
 template <typename op_desc_t, typename pd_t>
@@ -661,14 +690,12 @@ void test_fwd_pd_constructors(
     ASSERT_NO_THROW(test_pd = pd_t(pd.get()));
     // ctor w/ empty attr, should not throw
     test_fwd_pd_attr<op_desc_t, pd_t>(op_desc, eng);
-    // ctor w/ attr + output scales, depends on pd support
+    // following ctors w/ attrs may throw based on pd support
     test_fwd_pd_attr_oscale<op_desc_t, pd_t>(op_desc, eng, aa.oscale);
-    // ctor w/ attr + post op sum, depends on pd support
     test_fwd_pd_attr_po_sum<op_desc_t, pd_t>(op_desc, eng, aa.po_sum);
-    // ctor w/ attr + post op eltwise, depends on pd support
     test_fwd_pd_attr_po_eltwise<op_desc_t, pd_t>(op_desc, eng, aa.po_eltwise);
-    // ctor w/ attr + zero points, depends on pd support
     test_fwd_pd_attr_zp<op_desc_t, pd_t>(op_desc, eng, aa.zp);
+    test_fwd_pd_attr_scales<op_desc_t, pd_t>(op_desc, eng, aa.scales);
     // check allow empty, should not throw
     test_fwd_pd_allow_empty<op_desc_t, pd_t>(test_pd);
 }
@@ -678,48 +705,44 @@ template <typename op_desc_t, typename pd_t, typename hint_pd_t>
 void test_bwd_pd_attr(
         const op_desc_t &op_desc, const engine &eng, const hint_pd_t &hint) {
     dnnl::primitive_attr attr;
-    pd_t new_pd {};
-    EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr, eng, hint));
+    EXPECT_NO_THROW(pd_t pd(op_desc, attr, eng, hint));
 }
 
 template <typename op_desc_t, typename pd_t, typename hint_pd_t>
 void test_bwd_pd_attr_oscale(const op_desc_t &op_desc, const engine &eng,
         const hint_pd_t &hint, bool supports_oscale) {
     dnnl::primitive_attr attr_oscale;
-    attr_oscale.set_output_scales(0, {2.});
-    pd_t new_pd {};
+    attr_oscale.set_output_scales(0, {2.f});
     if (supports_oscale)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_oscale, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_oscale, eng, hint));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_oscale, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_oscale, eng, hint));
 }
 
 template <typename op_desc_t, typename pd_t, typename hint_pd_t>
 void test_bwd_pd_attr_po_sum(const op_desc_t &op_desc, const engine &eng,
         const hint_pd_t &hint, bool supports_po_sum) {
-    dnnl::primitive_attr attr_po_sum;
     dnnl::post_ops ops_sum;
     ops_sum.append_sum(1.1f);
+    dnnl::primitive_attr attr_po_sum;
     attr_po_sum.set_post_ops(ops_sum);
-    pd_t new_pd {};
     if (supports_po_sum)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_po_sum, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_sum, eng, hint));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_po_sum, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_sum, eng, hint));
 }
 
 template <typename op_desc_t, typename pd_t, typename hint_pd_t>
 void test_bwd_pd_attr_po_eltwise(const op_desc_t &op_desc, const engine &eng,
         const hint_pd_t &hint, bool supports_po_eltwise) {
-    dnnl::primitive_attr attr_po_eltwise;
     dnnl::post_ops ops_eltwise;
-    ops_eltwise.append_eltwise(1.0f, dnnl::algorithm::eltwise_relu, 0.f, 0.f);
+    ops_eltwise.append_eltwise(1.f, dnnl::algorithm::eltwise_relu, 0.f, 0.f);
+    dnnl::primitive_attr attr_po_eltwise;
     attr_po_eltwise.set_post_ops(ops_eltwise);
-    pd_t new_pd {};
     if (supports_po_eltwise)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_po_eltwise, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_eltwise, eng, hint));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_po_eltwise, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_eltwise, eng, hint));
 }
 
 template <typename op_desc_t, typename pd_t, typename hint_pd_t>
@@ -727,11 +750,18 @@ void test_bwd_pd_attr_zp(const op_desc_t &op_desc, const engine &eng,
         const hint_pd_t &hint, bool supports_zero_point) {
     dnnl::primitive_attr attr_zp;
     attr_zp.set_zero_points(DNNL_ARG_SRC, 0, {1});
-    pd_t new_pd {};
     if (supports_zero_point)
-        EXPECT_NO_THROW(new_pd = pd_t(op_desc, attr_zp, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(op_desc, attr_zp, eng, hint));
     else
-        EXPECT_ANY_THROW(new_pd = pd_t(op_desc, attr_zp, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_zp, eng, hint));
+}
+
+template <typename op_desc_t, typename pd_t, typename hint_pd_t>
+void test_bwd_pd_attr_scales(const op_desc_t &op_desc, const engine &eng,
+        const hint_pd_t &hint, bool supports_scales) {
+    dnnl::primitive_attr attr_scales;
+    attr_scales.set_scales(DNNL_ARG_SRC, 0, {2.f});
+    EXPECT_ANY_THROW(pd_t pd(op_desc, attr_scales, eng, hint));
 }
 
 template <typename op_desc_t, typename pd_t, typename hint_pd_t>
@@ -754,17 +784,29 @@ void test_bwd_pd_constructors(const op_desc_t &op_desc, const pd_t &pd,
     ASSERT_NO_THROW(test_pd = pd_t(pd.get()));
     // ctor w/ empty attr, should not throw
     test_bwd_pd_attr<op_desc_t, pd_t>(op_desc, eng, hint_pd);
-    // ctor w/ attr + output scales, depends on pd support
+    // following ctors w/ attrs may throw based on pd support
     test_bwd_pd_attr_oscale<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.oscale);
-    // ctor w/ attr + post op sum, depends on pd support
     test_bwd_pd_attr_po_sum<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.po_sum);
-    // ctor w/ attr + post op eltwise, depends on pd support
     test_bwd_pd_attr_po_eltwise<op_desc_t, pd_t>(
             op_desc, eng, hint_pd, aa.po_eltwise);
-    // ctor w/ attr + zero points, depends on pd support
     test_bwd_pd_attr_zp<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.zp);
+    test_bwd_pd_attr_scales<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.scales);
     // check allow empty, should not throw
     test_bwd_pd_allow_empty<op_desc_t, pd_t>(test_pd, hint_pd);
 }
 
+inline dnnl::stream make_stream(dnnl::engine engine,
+        dnnl::stream::flags flags = dnnl::stream::flags::default_flags) {
+#if DNNL_CPU_THREADING_RUNTIME == DNNL_RUNTIME_THREADPOOL
+    if (engine.get_kind() != dnnl::engine::kind::cpu)
+        return dnnl::stream(engine, flags);
+    dnnl::stream_attr stream_attr(dnnl::engine::kind::cpu);
+    stream_attr.set_threadpool(
+            dnnl::impl::threadpool_utils::get_active_threadpool());
+    return dnnl::stream(engine, flags, stream_attr);
+#else
+    return dnnl::stream(engine, flags);
+#endif
+}
+
 #endif
